"use strict";
/*
 * Copyright (c) 2020 Broadcom.
 * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
 *
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *   Broadcom, Inc. - initial API and implementation
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EndevorController = void 0;
const SettingsFacade_1 = require("./service/SettingsFacade");
const EndevorNodes_1 = require("./ui/tree/EndevorNodes");
const EndevorDataProvider_1 = require("./ui/tree/EndevorDataProvider");
class EndevorController {
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    constructor() {
        /**
         * This is the root node of Explorer for Endevor view. It is derived from [TreeItem](#TreeItem).
         * Its children are repository nodes.
         */
        this._rootNode = new EndevorNodes_1.EndevorNode(undefined);
        this.connections = new Map();
    }
    static get instance() {
        if (!this._instance) {
            this._instance = new EndevorController();
        }
        return this._instance;
    }
    get rootNode() {
        return this._rootNode;
    }
    set rootNode(value) {
        this._rootNode = value;
    }
    addRepository(repo, connectionLabel) {
        if (typeof repo.id === 'undefined') {
            repo.id = this.findNextId(connectionLabel);
        }
        const newRepoNode = new EndevorNodes_1.EndevorNode(repo);
        const conn = this.connections.get(connectionLabel);
        if (conn) {
            conn.loadRepository(repo);
        }
        this._rootNode.children.forEach((child) => {
            const entity = child.getEntity();
            if (entity && entity.getName() === connectionLabel) {
                child.children.push(newRepoNode);
            }
        });
    }
    getConnections() {
        return Array.from(this.connections.values());
    }
    addConnection(connection) {
        if (!connection.name) {
            connection.name = '';
        }
        this.connections.set(connection.name, connection);
        const newConnectionNode = new EndevorNodes_1.EndevorNode(connection);
        this._rootNode.children.push(newConnectionNode);
    }
    removeConnection(connectionName) {
        this.connections.delete(connectionName);
        this._rootNode.children.forEach((connection, index) => {
            if (connection.label === connectionName) {
                this._rootNode.children.splice(index, 1);
            }
        });
    }
    removeRepository(repoName, connectionLabel) {
        const conn = this.connections.get(connectionLabel);
        if (conn) {
            conn.getRepositoryMap().delete(repoName);
        }
        this._rootNode.children.forEach((connection, index) => {
            if (connection.label === connectionLabel) {
                this._rootNode.children[index].children = connection.children.filter((repo) => repo.label !== repoName);
            }
        });
    }
    updateRepositoryName(oldRepoName, newRepoName, connectionLabel) {
        const newMap = new Map();
        const conn = this.connections.get(connectionLabel);
        if (conn) {
            conn.getRepositoryMap().forEach((repo, name) => {
                if (name === oldRepoName) {
                    repo.setName(newRepoName);
                    newMap.set(newRepoName, repo);
                }
                else {
                    newMap.set(name, repo);
                }
            });
            conn.repositories = newMap;
        }
        const cnxIdx = this.rootNode.children.findIndex((node) => node.label === connectionLabel);
        const repoIdx = this.rootNode.children[cnxIdx].children.findIndex((repo) => repo.label === oldRepoName);
        this.rootNode.children[cnxIdx].children[repoIdx].label = newRepoName;
    }
    updateSettings() {
        SettingsFacade_1.SettingsFacade.updateSettings(this.getConnections());
    }
    updateNeedReloadInTree(parNeedReload, refreshTreeRoot) {
        refreshTreeRoot.updateNeedReload(parNeedReload);
    }
    /**
     * Loads the repositories(hosts) from settings.json into the model. It uses the `id` to find out if the loaded repository
     * already exists in the model and then load information from config into the model otherwise keeping the Endevor metadata.
     * Afterwards, it check if filters have been changed and reloads them as well.
     * This function also turns on the `needReload` flag in the repository nodes (in case Endevor instance was changed), and also root
     * node in case number of repositories has changed, or if some repository have changed.
     * In case `id` is not present in the settings.json, function [updateIDs](#EndevorController.updateIDs) is used to determine it
     * and store in the settings.json.
     */
    loadRepositories() {
        const connectionsFromSettings = new Map();
        SettingsFacade_1.SettingsFacade.listConnections().forEach((conn) => connectionsFromSettings.set(conn.name, conn.hosts));
        this.rootNode.needReload = false;
        if (this.rootNode.children.length !== this.connections.size) {
            this.rootNode.needReload = true;
        }
        connectionsFromSettings.forEach((hostList, connName) => {
            const connNode = EndevorController.instance.findNodeByConnectionName(connName);
            if (connNode) {
                const reposFromSettings = SettingsFacade_1.SettingsFacade.listRepositories(connName);
                const updatedRepos = new Map();
                reposFromSettings.forEach((settingsRepo) => {
                    const repoNode = EndevorController.instance.findNodeByRepoID(settingsRepo.id, connName);
                    let repoToKeep = settingsRepo;
                    if (repoNode) {
                        const modelRepo = repoNode.getRepository();
                        if (modelRepo) {
                            if (modelRepo.isEqual(settingsRepo)) {
                                this.rootNode.needReload = true;
                                if (!modelRepo.isSameInstance(settingsRepo)) {
                                    repoNode.needReload = !repoNode.hasChildren();
                                }
                                else {
                                    repoNode.needReload = true;
                                }
                                modelRepo.loadInfoFromConfig(settingsRepo);
                            }
                            repoToKeep = modelRepo;
                        }
                        this.checkAndReloadFilters(repoNode, repoToKeep);
                        repoNode.updateInfo();
                    }
                    else {
                        this.rootNode.needReload = true;
                    }
                    updatedRepos.set(repoToKeep.getName(), repoToKeep);
                });
                const currentRepos = updatedRepos;
                currentRepos.forEach((repo) => {
                    let profileLabel = repo.getProfileLabel();
                    if (!profileLabel) {
                        profileLabel = '';
                    }
                    EndevorController.instance.addRepository(repo, profileLabel);
                });
                this.updateIDs(connName);
            }
            else {
                const endevorDataProvider = new EndevorDataProvider_1.EndevorDataProvider();
                endevorDataProvider.addSession(connName);
            }
        });
    }
    isRepoInConnection(repoName, connectionLabel) {
        const conn = this.connections.get(connectionLabel);
        if (conn) {
            const repoMap = conn.getRepositoryMap();
            return repoMap.get(repoName) ? true : false;
        }
        return false;
    }
    findNodeByRepoID(id, connectionLabel) {
        if (id === undefined) {
            return undefined;
        }
        const connection = this.findNodeByConnectionName(connectionLabel);
        if (connection) {
            for (const node of connection.children) {
                const nodeRepo = node.getRepository();
                if (nodeRepo && nodeRepo.id === id) {
                    return node;
                }
            }
        }
        return undefined;
    }
    findNodeByConnectionName(name) {
        if (!name) {
            return undefined;
        }
        for (const node of this._rootNode.children) {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            if (node.getEntity() && node.getEntity().getName() === name) {
                return node;
            }
        }
    }
    /**
     * Function used to determine next available `id`.
     */
    findNextId(connectionLabel) {
        const conn = this.connections.get(connectionLabel);
        if (conn) {
            const repoMap = conn.getRepositoryMap();
            const iDArray = new Array(repoMap.size);
            iDArray.fill(true);
            repoMap.forEach((repo) => {
                if (repo.id !== undefined) {
                    iDArray[repo.id] = false;
                }
            });
            for (let i = 0; i < iDArray.length; i++) {
                if (iDArray[i]) {
                    return i;
                }
            }
            return iDArray.length;
        }
        return -1;
    }
    /**
     * In case `id` is not present in the settings.json, this function will determine it
     * and store in the settings.json.
     */
    updateIDs(connectionLabel) {
        const conn = this.connections.get(connectionLabel);
        if (conn) {
            const repoMap = conn.getRepositoryMap();
            let saveRepos = false;
            repoMap.forEach((repo) => {
                if (repo.id === undefined) {
                    repo.id = EndevorController.instance.findNextId(connectionLabel);
                    saveRepos = true;
                }
            });
            if (saveRepos) {
                this.updateSettings();
            }
        }
    }
    /**
     * Checks if current filters of the repository node - `repoNode` are different from the loaded ones in `loadedRepo`.
     * Along with updating the filters in model (`Repository`), it also updates the tree.
     * @param repoNode `EndevorNode` containing repository with current filters.
     * @param repo Repository loaded from settings.json
     */
    checkAndReloadFilters(repoNode, repo) {
        // eslint-disable-next-line @typescript-eslint/consistent-type-assertions
        const filtersNode = (repoNode.children.find((child) => 
        // eslint-disable-next-line @typescript-eslint/consistent-type-assertions
        child.isFiltersNode()));
        if (!filtersNode) {
            return;
        }
        const newChildren = new Map();
        repo.filters.forEach((filter) => {
            let newNode = filtersNode.findFilterNode(filter);
            if (newNode) {
                newNode.updateInfo();
                newNode.needReload = !newNode.hasChildren();
            }
            else {
                newNode = new EndevorNodes_1.FilterNode(filter);
            }
            newChildren.set(filter.getUri(), newNode);
        });
        filtersNode.children = Array.from(newChildren.values());
        filtersNode.needReload = false;
    }
}
exports.EndevorController = EndevorController;
//# sourceMappingURL=EndevorController.js.map