"use strict";
/*
 * Copyright (c) 2020 Broadcom.
 * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
 *
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *   Broadcom, Inc. - initial API and implementation
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const RetrieveElement_1 = require("../../../commands/RetrieveElement");
const globals_1 = require("../../../globals");
const Element_1 = require("../../../model/Element");
const Repository_1 = require("../../../model/Repository");
const RetrieveElementService_1 = require("../../../service/RetrieveElementService");
const EndevorNodes_1 = require("../../../ui/tree/EndevorNodes");
// Explicitly show NodeJS how to find VSCode (required for Jest)
process.vscode = vscode;
describe('Test function retrieveElement', () => {
    // Mock the workspace folder so it is selected/unselected, as needed
    const mockWorkspaceFolders = ['testFolder'];
    Object.defineProperty(vscode.workspace, 'workspaceFolders', {
        value: mockWorkspaceFolders,
    });
    // Mock vscode's progress function
    const mockWithProgress = jest.fn();
    Object.defineProperty(vscode, 'ProgressLocation', {
        value: { Notification: 15 },
    });
    Object.defineProperty(vscode.window, 'withProgress', {
        value: mockWithProgress,
    });
    // Mock the elements, nodes, and repo
    const mockRetrieveElement = jest.fn();
    const testRetrieveEltService = new RetrieveElementService_1.RetrieveElementService();
    Object.defineProperty(testRetrieveEltService, 'retrieveElement', {
        value: mockRetrieveElement,
    });
    const testRepo = new Repository_1.Repository('testRepo', 'https://example.com:1234', 'testUser', 'testPass', 'testRepo', 'testConnLabel');
    const testIElements = [
        {
            elmName: 'elmTest1',
            fullElmName: 'elmTest1',
            elmVVLL: '1100',
            envName: 'envTest',
            sysName: 'sysTest',
            sbsName: 'sbsTest',
            stgNum: '1',
            typeName: 'COBOL',
        },
        {
            elmName: 'elmTest2',
            fullElmName: 'elmTest2',
            elmVVLL: '1100',
            envName: 'envTest',
            sysName: 'sysTest',
            sbsName: 'sbsTest',
            stgNum: '1',
            typeName: 'COBOL',
        },
    ];
    const testElements = [
        new Element_1.Element(testRepo, testIElements[0]),
        new Element_1.Element(testRepo, testIElements[1]),
    ];
    const qualifier1 = {
        element: testElements[0].elmName,
        env: 'envTest',
        stage: '1',
        subsystem: 'sbsTest',
        system: 'sysTest',
        type: 'COBOL',
    };
    const qualifier2 = {
        element: testElements[1].elmName,
        env: 'envTest',
        stage: '1',
        subsystem: 'sbsTest',
        system: 'sysTest',
        type: 'COBOL',
    };
    const testEndevorElementNodes = [
        new EndevorNodes_1.EndevorElementNode(testElements[0], qualifier1),
        new EndevorNodes_1.EndevorElementNode(testElements[1], qualifier2),
    ];
    // All spies are listed here
    const openDocumentSpy = jest.spyOn(vscode.workspace, 'openTextDocument');
    const retrieveElementErrorSpy = jest.spyOn(testRetrieveEltService, 'processRetrieveElementError');
    const loggerErrorSpy = jest.spyOn(globals_1.logger, 'error');
    beforeEach(() => {
        // Redefine mocks, because we clear them after each run
        mockWithProgress.mockImplementation((progLocation, callback) => __awaiter(void 0, void 0, void 0, function* () {
            const ProgressLocation = {
                Notification: 15,
                report: jest.fn(),
            };
            return yield callback(ProgressLocation);
        }));
        mockRetrieveElement.mockReturnValue('test/elt/path');
    });
    afterEach(() => {
        // This is here to clear the spies
        jest.clearAllMocks();
    });
    test('Should preview one element in the text editor, if passed in as arg', () => __awaiter(void 0, void 0, void 0, function* () {
        yield RetrieveElement_1.retrieveElement(testEndevorElementNodes[0], [], testRetrieveEltService);
        expect(openDocumentSpy).toBeCalledWith('test/elt/path');
    }));
    test('Should preview many elements in the text editor, if passed in as array', () => __awaiter(void 0, void 0, void 0, function* () {
        mockRetrieveElement.mockReturnValueOnce('test/elt/path/first');
        yield RetrieveElement_1.retrieveElement(null, testEndevorElementNodes, testRetrieveEltService);
        expect(openDocumentSpy).toBeCalledTimes(2);
        expect(openDocumentSpy.mock.calls[0]).toEqual(['test/elt/path/first']);
        expect(openDocumentSpy.mock.calls[1]).toEqual(['test/elt/path']);
    }));
    test('Should throw an error, if element cannot be retrieved from Endevor', () => __awaiter(void 0, void 0, void 0, function* () {
        mockRetrieveElement.mockRejectedValueOnce('test error!');
        yield RetrieveElement_1.retrieveElement(testEndevorElementNodes[0], [], testRetrieveEltService);
        expect(retrieveElementErrorSpy).toBeCalledWith('test error!');
    }));
    test('Should throw an error, if no folder is open in the workspace', () => __awaiter(void 0, void 0, void 0, function* () {
        mockWorkspaceFolders.pop();
        yield RetrieveElement_1.retrieveElement(testEndevorElementNodes[0], [], testRetrieveEltService);
        expect(loggerErrorSpy).toBeCalledWith('Specify workspace before retrieving elements');
    }));
});
//# sourceMappingURL=retrieveElement-test.js.map