"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Profiles = void 0;
/* eslint-disable @typescript-eslint/consistent-type-assertions */
const endevor_for_zowe_cli_1 = require("@broadcom/endevor-for-zowe-cli");
const imperative_1 = require("@zowe/imperative");
const os = require("os");
const path = require("path");
const url_1 = require("url");
const vscode = require("vscode");
const globals_1 = require("../globals");
class Profiles {
    constructor(log) {
        this.log = log;
        this.allProfiles = [];
        this.validateAndParseUrl = (newUrl) => {
            let url;
            const validProtocols = ['https', 'http'];
            const validationResult = {
                host: undefined,
                port: undefined,
                protocol: undefined,
                valid: false,
            };
            try {
                url = new url_1.URL(newUrl);
            }
            catch (error) {
                return validationResult;
            }
            // overkill with only one valid protocol, but we may expand profile types and protocols in the future?
            if (!validProtocols.some((validProtocol) => url.protocol.includes(validProtocol))) {
                return validationResult;
            }
            else {
                validationResult.protocol = url.protocol.replace(':', '');
            }
            if (!url.port.trim()) {
                return validationResult;
            }
            else {
                validationResult.port = Number(url.port);
            }
            validationResult.host = url.hostname;
            validationResult.valid = true;
            return validationResult;
        };
    }
    static createInstance(log) {
        return __awaiter(this, void 0, void 0, function* () {
            Profiles.loader = new Profiles(log);
            yield Profiles.loader.refresh();
            return Profiles.loader;
        });
    }
    static getInstance() {
        return Profiles.loader;
    }
    loadNamedProfile(name) {
        for (const profile of this.allProfiles) {
            if (profile.name === name && profile.type === 'endevor') {
                return profile;
            }
        }
        throw new Error('Could not find profile named: ' + name + '.');
    }
    getDefaultProfile() {
        return this.defaultProfile;
    }
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            this.allProfiles = [];
            const profileManager = this.getEndevorCliProfileManager();
            const endevorProfiles = (yield (yield profileManager).loadAll()).filter((profile) => {
                return profile.type === 'endevor';
            });
            if (endevorProfiles && endevorProfiles.length > 0) {
                this.allProfiles.push(...endevorProfiles);
                let defaultProfile;
                try {
                    defaultProfile = yield (yield profileManager).load({
                        loadDefault: true,
                    });
                    this.defaultProfile = defaultProfile ? defaultProfile : undefined;
                }
                catch (error) {
                    globals_1.logger.error(error.message);
                }
            }
        });
    }
    listProfiles() {
        return this.allProfiles;
    }
    getUrl(urlInputBox) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                urlInputBox.onDidHide(() => {
                    resolve(urlInputBox.value);
                });
                urlInputBox.onDidAccept(() => {
                    if (this.validateAndParseUrl(urlInputBox.value).valid) {
                        resolve(urlInputBox.value);
                    }
                    else {
                        urlInputBox.validationMessage =
                            'Please enter a valid URL in the format http(s)://url:port.';
                    }
                });
            });
        });
    }
    createNewConnection(profileName) {
        return __awaiter(this, void 0, void 0, function* () {
            let rejectUnauthorize;
            let options;
            if (!profileName || profileName === '') {
                globals_1.logger.info('No valid value for new profile name. Operation Cancelled');
                return undefined;
            }
            for (const profile of this.allProfiles) {
                if (profile.name === profileName) {
                    globals_1.logger.error('Profile name already exists. Please create a profile using a different name');
                    return undefined;
                }
            }
            const urlInputBox = vscode.window.createInputBox();
            urlInputBox.ignoreFocusOut = true;
            urlInputBox.placeholder = 'http(s)://url:port';
            urlInputBox.prompt =
                "Enter an Endevor URL in the format 'http(s)://url:port'.";
            urlInputBox.show();
            const endevorURL = yield this.getUrl(urlInputBox);
            urlInputBox.dispose();
            if (!endevorURL) {
                globals_1.logger.info('No valid value for Endevor URL. Operation Cancelled');
                return undefined;
            }
            const endevorUrlParsed = this.validateAndParseUrl(endevorURL);
            options = {
                placeHolder: 'Username',
                prompt: 'Enter the username for the connection.',
            };
            const userName = yield vscode.window.showInputBox(options);
            if (userName === undefined) {
                globals_1.logger.info('No valid value for username. Operation Cancelled');
                return;
            }
            options = {
                password: true,
                placeHolder: 'Password',
                prompt: 'Enter the password for the connection.',
            };
            const passWord = yield vscode.window.showInputBox(options);
            if (passWord === undefined) {
                globals_1.logger.info('No valid value for password. Operation Cancelled');
                return;
            }
            const quickPickOptions = {
                canPickMany: false,
                ignoreFocusOut: true,
                placeHolder: 'Reject Unauthorized Connections',
            };
            const selectRU = [
                'True - Reject connections with self-signed certificates',
                'False - Accept connections with self-signed certificates',
            ];
            const ruOptions = Array.from(selectRU);
            const chosenRU = yield vscode.window.showQuickPick(ruOptions, quickPickOptions);
            if (chosenRU === ruOptions[0]) {
                rejectUnauthorize = true;
            }
            else if (chosenRU === ruOptions[1]) {
                rejectUnauthorize = false;
            }
            else {
                globals_1.logger.info('No valid value for Reject Unauthorized. Operation Cancelled');
                return undefined;
            }
            const connection = {
                host: endevorUrlParsed.host,
                name: profileName,
                password: passWord,
                port: endevorUrlParsed.port,
                protocol: endevorUrlParsed.protocol,
                rejectUnauthorized: rejectUnauthorize,
                user: userName,
            };
            let newProfile;
            try {
                newProfile = yield this.saveProfile(connection, connection.name, 'endevor');
                yield this.createBasicEndevorSession(newProfile);
                globals_1.logger.info(`Profile ${profileName} was created.`);
                yield this.refresh();
                return profileName;
            }
            catch (error) {
                globals_1.logger.error('Error saving profile: ', error.message);
            }
        });
    }
    createBasicEndevorSession(profile) {
        return __awaiter(this, void 0, void 0, function* () {
            globals_1.logger.trace(`Creating an Endevor session from the profile named ${profile.name}`);
            return new imperative_1.Session({
                base64EncodedAuth: profile.auth,
                basePath: profile.basePath,
                hostname: profile.host,
                password: profile.password,
                port: profile.port,
                rejectUnauthorized: profile.rejectUnauthorized,
                type: 'basic',
                user: profile.user,
            });
        });
    }
    promptCredentials(sessName) {
        return __awaiter(this, void 0, void 0, function* () {
            let userName;
            let passWord;
            let options;
            const loadProfile = this.loadNamedProfile(sessName);
            const loadSession = loadProfile.profile;
            if (!loadSession.user) {
                options = {
                    placeHolder: 'User Name',
                    prompt: 'Enter the user name for the connection',
                };
                userName = yield vscode.window.showInputBox(options);
                if (!userName) {
                    globals_1.logger.error('Please enter your z/OS username. Operation Cancelled');
                    return;
                }
                else {
                    loadSession.user = userName;
                }
            }
            if (!loadSession.password) {
                passWord = loadSession.password;
                options = {
                    password: true,
                    placeHolder: 'Password',
                    prompt: 'Enter a password for the connection',
                };
                passWord = yield vscode.window.showInputBox(options);
                if (!passWord) {
                    globals_1.logger.error('Please enter your z/OS password. Operation Cancelled');
                    return;
                }
                else {
                    loadSession.password = passWord.trim();
                }
            }
            const updSession = yield this.createBasicEndevorSession(loadSession);
            return [
                updSession.ISession.user,
                updSession.ISession.password,
                updSession.ISession.base64EncodedAuth,
            ];
        });
    }
    getEndevorCliProfileManager() {
        return __awaiter(this, void 0, void 0, function* () {
            let profileManager = this.endevorProfileManager;
            if (!profileManager) {
                try {
                    yield imperative_1.CliProfileManager.initialize({
                        configuration: endevor_for_zowe_cli_1.EndevorProfilesConfig,
                        profileRootDirectory: path.join(this.getZoweDir(), 'profiles'),
                        reinitialize: false,
                    });
                    profileManager = new imperative_1.CliProfileManager({
                        profileRootDirectory: path.join(this.getZoweDir(), 'profiles'),
                        type: 'endevor',
                    });
                    this.endevorProfileManager = profileManager;
                }
                catch (error) {
                    globals_1.logger.error('Failed to load Imperative Profile Manager.', error.message);
                }
            }
            return profileManager;
        });
    }
    getZoweDir() {
        imperative_1.ImperativeConfig.instance.loadedConfig = {
            defaultHome: path.join(os.homedir(), '.zowe'),
            envVariablePrefix: 'ZOWE',
        };
        return imperative_1.ImperativeConfig.instance.cliHome;
    }
    saveProfile(ProfileInfo, ProfileName, ProfileType) {
        return __awaiter(this, void 0, void 0, function* () {
            const endevorProfile = yield (yield this.getEndevorCliProfileManager()).save({
                name: ProfileName,
                profile: ProfileInfo,
                type: ProfileType,
            });
            return endevorProfile.profile;
        });
    }
}
exports.Profiles = Profiles;
//# sourceMappingURL=Profiles.js.map